package com.beadlefox.xmlsocket;

/*

Copyright (c) 2007 Jason R. Weiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/


import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;

public class ClientThread extends Thread implements RandomEventListener
{
	private static final String xmlSocketLineTerminator = "\r\0";
	
	private volatile boolean runThread = true;

	private Socket socket;

	private SimpleServer server;

	private RandomEventGenerator generator;

	private int clientId;

	private BufferedReader in = null;

	private PrintWriter out = null;

	public ClientThread(SimpleServer server, RandomEventGenerator generator,
			Socket socket, int clientId)
	{
		//
		// Track the server so that a shutdown command can be communicated
		//
		this.server = server;

		//
		// Track the random event generator so that we can register for events
		// and unregister for events on behalf of the Flex client
		//
		this.generator = generator;

		//
		// Track this clients socket so that we can read and write data
		// 
		this.socket = socket;

		//
		// Keep track of our unique client ID assigned by the server
		//
		this.clientId = clientId;
	}

	public void run()
	{
		System.out.println("Client ID " + clientId + " thread starting...");

		try
		{
			//
			// Capture our input and output streams so that the client
			// can read and write data
			//
			in = new BufferedReader(new InputStreamReader(socket
					.getInputStream()));
			out = new PrintWriter(new OutputStreamWriter(socket
					.getOutputStream()));

			//
			// At this point, we can read for input and reply with appropriate
			// output. Run in a loop until runThread is set to false when the
			// client issues a <quit/> or <shutdown/> command.
			//
			while (runThread)
			{
				//
				// Read the incoming stream sent from the client
				//
				String xml = in.readLine();
				System.out.println("Client ID " + clientId
						+ " XML Raw Request Received:\n===========\n" + xml + "\n===========\n");

				//
				// In case of a loitering null byte, strip it out because
				// it only causes parsing problems
				//
				byte[] bytes = xml.getBytes();
				StringBuffer processedXml = new StringBuffer();

				for (int i = 0; i < bytes.length; i++)
				{
					if (bytes[i] > 0)
					{
						processedXml.append((char) bytes[i]);
					} else
					{
						System.out.println("Skipping byte #" + i + " because it is null");
					}
				}
				System.out.println("Processed XML: " + processedXml.toString());

				//
				// Quick and dirty way to determine what action the
				// client has requested from the server
				//
				String action = processedXml.toString().toLowerCase().substring(1, 5);
				System.out.println("Action Request: " + action);

				if (action.equals("shut"))
				{
					//
					// Shutdown the server in its entirety
					//
					System.out.println("Client ID " + clientId
							+ " is terminating the server");
					runThread = false;

					out.write(generateXmlResponse(true, ""));
					out.flush();
					server.shutdownServer();
				}
				if (action.equals("quit"))
				{
					//
					// Client is exiting; terminate this thread
					//
					System.out.println("Client ID " + clientId + " is leaving");
					runThread = false;

					out.write(generateXmlResponse(true, ""));
					out.flush();
				} else if (action.equals("regi"))
				{
					//
					// Client is registering for an event
					//
					int eventNameStart = xml.indexOf("=\"");
					int eventNameEnd = xml.lastIndexOf("\"");
					String eventName = xml.substring(eventNameStart + 2,
							eventNameEnd);
					System.out.println("Client ID " + clientId
							+ " has registered for event " + eventName);

					//
					// Notify the random event generator an interest
					// in events on behalf of the Flex client. In
					// an enterprise environment, this could effectively
					// be done by registering with a JMS topic or queue
					// and implementing an onMessage method to process
					// notifications from the JMS server
					//
					generator.registerEventListener(eventName, this);

					out.write(generateXmlResponse(true, ""));
					out.flush();
				}
			}
		} catch (IOException ioe)
		{
			System.err.println(ioe.getMessage());
			ioe.printStackTrace();
			return;
		} finally
		{
			//
			// The thread is exiting; properly close any open resources
			//
			try
			{
				if (null != in)
					in.close();

				if (null != out)
					out.close();

				if (null != socket)
					socket.close();

				System.out.println("Thread successfully stopped");
			} catch (IOException e)
			{
				System.err
						.println("Unexpected exception cleaning up client thread "
								+ clientId + ": " + e.getMessage());
				e.printStackTrace();
			}
		}
	}

	public String generateXmlResponse(boolean successFlag, String errorMessage)
	{
		return "<response success=\"" + successFlag + "\""
				+ ((!successFlag) ? "error=\"" + errorMessage + "\"" : "")
				+ "/>" + xmlSocketLineTerminator;
	}

	public void onEvent(String eventName)
	{
		//
		// Random event received- notify the client!
		//
		System.out.println("onEvent Receieved for Client ID " + clientId
				+ " for event " + eventName);

		StringBuffer xmlEvent = new StringBuffer("<event name=\"" + eventName
				+ "\">\n");
		xmlEvent.append("\t<property name=\"timestamp\" value=\""
				+ System.currentTimeMillis() + "\"/>\n");
		xmlEvent.append("</event>" + xmlSocketLineTerminator);

		//
		// Push the event out to the client
		//
		out.write(xmlEvent.toString());
		out.flush();
	}

}
