package com.beadlefox.xmlsocket;

/*

Copyright (c) 2007 Jason R. Weiss

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/

import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;

public class SimpleServer
{
	public static final int SOCKET_PORT = 5974;

	private volatile boolean runServer = true;

	private ServerSocket socket;
	
	private RandomEventGenerator generator;
	
	public SimpleServer()
	{
		//
		// Create an instance of a Random Event Generator
		//
		generator = new RandomEventGenerator();
		generator.start();
	}

	public static void main(String[] args)
	{
		new SimpleServer().launchServer();
	}

	public void shutdownServer()
	{
		//
		// Set a flag that indicates any IOException
		// is expected because we are shutting down
		// the server and no longer accepting client
		// requests
		//
		runServer = false;
		
		//
		// Stop the generator and interrupt it
		// if the thread is asleep so it can terminate
		//
		generator.haltEventGeneration();
		generator.interrupt();

		//
		// Now, close the socket completely
		//
		if (null != socket)
		{
			try
			{
				socket.close();
			} catch (IOException e)
			{
				//
				// Ignore this for this demo
				//
			}
		}
		System.out.println("Server has shutdown");
	}

	public void launchServer()
	{
		Socket clientSocket;
		int nextAvailableClientId = 0;

		try
		{
			//
			// ServerSocket waits for requests to come in over the network
			// at the assigned location (port). All this step does is
			// assign the port that the server should listen too; it doesn't
			// actually tell the server to start listening yet
			//
			socket = new ServerSocket(SOCKET_PORT);

			System.out.println("Simple Server listening on port " + SOCKET_PORT
					+ " for client requests");

			while (runServer)
			{
				//
				// With the invocation of the accept() method, we are now
				// effectively turning on the server by telling it that if
				// and when it hears a connection request it should permit
				// that request to come through.
				//
				clientSocket = socket.accept();

				//
				// accept() will block until a client connects to the server.
				// If execution reaches this point, then it means that a client
				// socket has been accepted by the server. Now it is time
				// to process the request.
				//

				//
				// Each thread will be assigned a unique ID for logging purposes
				// so we can see multiple clients being processed.
				//
				nextAvailableClientId++;

				//
				// For each client, we will start a thread to service the
				// client requests. Starting a thread lets our server accept
				// multiple connections- once this thread is started, the
				// server resumes listening for connection requests
				// 
				new ClientThread(this, generator, clientSocket, nextAvailableClientId)
						.start();
			}
		} catch (IOException ioe)
		{
			if (runServer)
			{
				System.err.println(ioe.getMessage());
				ioe.printStackTrace();
				System.exit(1);
			} else
			{
				//
				// This exception is a result of
				// shutting down the server- ignore it
				//
			}
		}
	}
}
